var chatId = getCookie('chatId');
var backendURL = 'vchat-backend-mysqli.php';
var lastTime = 0;
var recipientSel, msgForm, msgDiv, msgs, info, formDiv;
var isTyping = false;
var refreshInterval = null;
var typingInterval = null;
var refreshTime = 4000;
var isTypingTime = 4000;
var processing = false;
var maxId = 0;
var loggedOut = false;

$(document).ready(function() {		
	recipientSel = document.getElementById('recipient');
	formDiv = document.getElementById('formDiv');
	msgForm = document.getElementById('messageForm');
	msgDiv = document.getElementById('message');
	msgs = document.getElementById('messages');
	info = document.getElementById('info');

	$("form#messageForm").submit(function(){		
		onFormSubmit();
		return false;
	});	
	$("#message").keyup(function() { onMessageKeyUp(); });
	$("#recipient").change(function () {
		$("#private").attr('disabled', $("#recipient").val() == '');
		msgDiv.focus();
	});	
	$("#private").click(function() { msgDiv.focus(); });	
	$("#myMsgs").click(function() { traverseMessages(); });	
	$("#private").attr('disabled', true);
	$("#logout").click(function() { logout();});

	formDiv.style.height = msgDiv.offsetTop + msgDiv.offsetHeight + 'px';
	window.onresize = onWindowResize;
	onWindowResize();
	msgDiv.focus();
	
	updateMessages();
});

function onWindowResize() {
	msgForm.style.margin = 0;
	msgForm.style.paddingLeft = 2 + 'px';
	msgForm.style.paddingRight = 2 + 'px';
	msgDiv.style.width = msgForm.offsetWidth - 14 - recipientSel.offsetWidth + 'px';
	
	msgs.style.height = document.body.clientHeight - msgForm.offsetHeight - info.offsetHeight - 6 + 'px';
}

function onFormSubmit() {
	msg = $.trim($("#message").val());
	if(msg != '') {			
		$("#message").attr('disabled', true);
		$.post(backendURL,
			{
				action: 'insertMessage',
				message: msg, 
				recipientId: $("#recipient").val(),
				private: $("#private").attr('checked'),
				chatId: chatId
			}, function(xml) {					
				onInsertMessageResult(xml);
			}
		);
	}
}

function logout() {
	loggedOut = true;
	$.ajax({
		async: false,
		type: "POST",
	    url: backendURL,
	    data: { action: 'logout', chatId: chatId },
	    dataType: "xml",
	    success: function(xml) {
			onLogoutResult(xml);
		}
	});
	return false;
}

function onMessageKeyUp() {
	isTyping = true;
	clearInterval(typingInterval);
	typingInterval = setInterval("resetIsTyping()", isTypingTime);
}

function resetIsTyping() {
	isTyping = false;
	clearInterval(typingInterval);
}

function updateMessages() {
	processing = true;
	$.post(backendURL,
		{
	     	action: 'refresh',
	     	chatId: chatId,
	     	lastTime: lastTime,
	     	isTyping: isTyping
		}, function(xml) {					
			onUpdateMessageResult(xml);
		}
	);
}

function reportFlag(status, message) {
	resetFlag();

	switch (status) {
		case "1":
			$("#flag").addClass('flag-ok');
			break;
		case "2":
			$("#flag").addClass('flag-error');	
			if(message != null && message != '') {
				str = '<div class="error">' + message + '</div>';			
				$("#messages").prepend(str);
			}
			break;
	}
	setTimeout('resetFlag()', 2000);
}

function resetFlag() {
	$("#flag").removeClass('flag-error');
	$("#flag").removeClass('flag-ok');
	$("#flag").text('');
}

function onInsertMessageResult(xml) {	
	$("#message").attr('disabled', false);
	status = $("status", xml).text();
	message = '';
	if(status == "1") {
		$("#message").val('');
		if(!processing) updateMessages();
	} else {
		message = $("error", xml).text();
	}
	reportFlag(status, message);	
	msgDiv.blur();
	msgDiv.focus();	
}

function onLogoutResult(xml) {
	status = $("status", xml).text();
	if(status == "1") {
		showLoginPage('logout');
	} else {
		reportFlag(status, $("error", xml).text());
	}
}

function showLoginPage(action) {
	expireDate = new Date();
	expireDate.setSeconds(expireDate.getSeconds() - 15);		
	setCookie('chatId', '', expireDate);		
	window.location = 'chat-login.html?action=' + action;
}

function onUpdateMessageResult(xml) {	
	if(!processing || loggedOut) return;
	
	expireDate = new Date();
	expireDate.setSeconds(expireDate.getSeconds() + 30);		
	setCookie('chatId', chatId, expireDate);

	typingStr = '';	
	status = $("sysInfo/status", xml).text();
	switch(status) {
		case "1":	
			_lastTime = parseFloat($("sysInfo/lastTime",xml).text());
			if(_lastTime > lastTime) {			
				members = new Object();		
				$("members/member", xml).each(function(i){
					_chatId = $(this).attr('chatId');
					_chatMember = new Object();
					_chatMember['chatId'] = _chatId;
					_chatMember['color'] = $(this).attr('color');
					_chatMember['nick'] = $(this).text();
					_chatMember['inChat'] = $(this).attr('inChat');	
					_chatMember['isTyping'] = $(this).attr('isTyping');	
					_chatMember['userId'] = $(this).attr('userId');	
					members[_chatId] = _chatMember;	
	
					if(lastTime == 0 && _chatMember['inChat'] == "true") addChatMember(_chatMember);
					if(_chatMember['isTyping'] == "true" && _chatMember['chatId'] != chatId) {			    	
					    typingStr += '<span style="color:' + _chatMember['color'] + '; '+ (_chatMember['userId'] != null ? 'font-weight: bold;':'') + '">' + _chatMember['nick'] + '</span>, ';
					}
				});
		
				$("messages/*", xml).each(function(i){
					   switch(this.nodeName) {
						   	case 'sysMessage':
						   		addSysMessage(members, this);
						   		break;			   		
						   	case 'memMessage':
						   		addMemMessage(members, this);
						   		break;				 
					   }
				});
				
				lastTime = _lastTime;
			}
			break;
		case "3":
			showLoginPage('expire');
			break;
		default:		
			reportFlag(status, $("sysInfo/error", xml).text());
			clearInterval(refreshInterval);
			break;
	}
	typingStr = typingStr.substr(0, typingStr.length - 2)
	$("#typingUsers").html(typingStr);
	
	setTimeout("updateMessages()", refreshTime);
	processing = false;
}

function traverseMessages() {
	onlyMyMessages = $("#myMsgs").attr('checked');

	$("#messages > div:not([@id:contains('myDiv_')])").each(function(i){
		if(onlyMyMessages) {
			$(this).hide();
		} else {
			$(this).show();
		}
	});
}

function onMemberClick(_chatId) {
	for(i = 0; i < recipientSel.options.length; i++) {
		if(recipientSel.options[i].value == _chatId) {
			recipientSel.selectedIndex = i;
			msgDiv.focus();
			return;
		}
	}
}

function addChatMember(chatMember) {
	id = 'peopleInChat_' + chatMember['chatId'];
	if(chatMember['chatId'] != chatId) {
		newOption = new Option(chatMember['nick'], chatMember['chatId']);		
		try { recipientSel.add(newOption, recipientSel.options[recipientSel.options.length]); }
		catch(ex) { recipientSel.add(newOption); }	
		newOption.style.color = chatMember['color'];	
	} else if(lastTime == 0) {		
		$("#logout").before('<span style="color:' + chatMember['color'] + ';' + (_chatMember['userId'] != null ? ' font-weight: bold;':'') + '">' + chatMember['nick'] + '</span>' + '&nbsp;');
	}
	
	if($('#' + id).size() == 0) {
		_str = '';
		_str += '<span onclick="onMemberClick(\'' + chatMember['chatId'] + '\')" id="' + id + '" style="cursor: pointer; display: block; color:' + chatMember['color'] + ';';
		if(chatMember['userId'] > 0) _str += ' font-weight: bold;';	
		_str += '">' + chatMember['nick'];		
		if(chatMember['userId'] > 0) _str += ' <a title="Zobrazit garáž užitele ' + chatMember['nick'] + '" href="http://www.velorexy.cz/?loc1=103&user_id='+ chatMember['userId'] + '" onclick="window.open(this.href); return false;">G</a>';
		_str += '</span>';
		$('#peopleInChat').append(_str);
	}
}

function removeChatMember(chatMember) {
	$('#peopleInChat_' + chatMember['chatId']).remove();
	for(i = 0; i < recipientSel.options.length; i++) {
		if(recipientSel.options[i].value == chatMember['chatId']) {
			recipientSel.options[i] = null;
			return;
		}
	}
}

function addSysMessage(members, msg) {
	_time = parseFloat($(msg).attr('time'));
	if(_time <= lastTime) return;
	_chatId = $(msg).attr('chatId');
	_date = new Date();
	_formatedDate = formatDate(_time);	
	_chatMember = members[_chatId];

	if(_chatMember != null) {
		msgType = $(msg).attr('type');
		isUser = $(msg).attr('isUser');
		
		str = "<div class=\"system\"><span class=\"time\">[" + _formatedDate + "]</span> " + "<span style='";
		if(isUser == '1') str += " font-weight: bold;";
		str += " color:" + _chatMember['color'] + "'>" + _chatMember['nick']+ "</span> ";
		switch(msgType) {
			case 'in':
				str += " vstoupil do chatu";	
				if(lastTime != 0) addChatMember(_chatMember);
				break;
			case 'out':
				str += " opustil chat";
				if(lastTime != 0) removeChatMember(_chatMember);
				break;
			case 'expire':
				str += " byl pro neaktivnost odhlášen";
				if(lastTime != 0) removeChatMember(_chatMember);
				break;
		}		
		str += "</div>";	

		$("#messages").prepend(str);
	}
}

function addMemMessage(members, msg) {	
	_time = parseFloat($(msg).attr('time'));
	if(_time <= lastTime) return;
	_senderId = $(msg).attr('senderChatId');
	_recipientId = $(msg).attr('recipiendChatId');
	_date = new Date();
	_formatedDate = formatDate(_time);	
	_senderMember = members[_senderId];
	
	if(_senderMember != null) {
		_divId = maxId++;
		_private = $(msg).attr('private');
		
		str = '<div';	
		if(_recipientId == null || _senderId == chatId || _recipientId == chatId) str += ' id="myDiv_' +_divId + '"';		
		if(_private) str += ' class="private"';
		str += ' style="color: ' + _senderMember['color'] + ';"';
		str += '>';
		str += "<span class=\"time\">[" + _formatedDate + "]</span> " + "<span style='" + (_senderMember['userId'] != null ? 'font-weight: bold; ':'') + "color:"+_senderMember['color']+"'>" + _senderMember['nick']+ "</span>";			
		
		if(_recipientId != null) {
			_recipientMember = members[_recipientId];
			if(_recipientMember != null) {
				str += "-><span style='" + (_recipientMember['userId'] != null ? 'font-weight: bold; ':'') + "color:"+_recipientMember['color']+"'>" + _recipientMember['nick']+ "</span>";
			}
		}

		str += ': ' + $(msg).text();
		str += "</div>";	
		
		$("#messages").prepend(str);
	} 
}

function formatDate(time) {
	_date = new Date();
	_date.setTime(parseInt(time) * 1000);
	h = _date.getHours();
	m = _date.getMinutes();
	s = _date.getSeconds();
	return (h < 10 ? '0':'') + h + ":" + (m < 10 ? '0':'') + m + ":" + (s < 10 ? '0':'') + s;
}
